#include "wiiuse.h"
#include "actionset.h"
#include <iostream>
#include <fstream>
#include <X11/Xlib.h>
#include <sys/time.h>
#include <cmath>

using namespace std;

#define STICK_MAGNITUDE_THRESHOLD 30 /* 0.2 actually, will divide by DENOMINATOR in code; same below */
#define SHOULDER_MAGNITUDE_THRESHOLD 30
#define DENOMINATOR 100
#define STICK_ANGLE_THRESHOLD 65 /* degrees away from actuall 4 point compass direction the stick 
                                    can be and still activate the corresponding event */
#define ROLL_THRESHOLD 15 /* in degrees, same for PITCH_THRESHOLD */
#define PITCH_THRESHOLD 10
#define FLICK_THRESHOLD 4 /* amount of gforce necessary to trigger the flick event */
#define NC_FLICK_THRESHOLD 2 /* amount of gforce necessary to trigger the flick event on the nunchuk */
#define FRAME_DURATION 10000

long int get_time();
void handle_ctrl_status(struct wiimote_t* wm, int attachment, int speaker, int ir, int led[4], float battery_level);
void handle_event(struct wiimote_t* wm);
void handle_disconnect(wiimote* wm) {
	printf("\n\n--- DISCONNECTED [wiimote id %i] ---\n", wm->unid);
}
void process_mouse_instructions();
void toggle_wiimote_motion_sensing();

void update_led_timer(wiimote_t* wm, long int time );

Display *display;
Window winRoot;
ActionSet actions; // needs to be accessible from handle_event()
CC_ActionSet ccactions; // needs to be accessible from handle_event()
NC_ActionSet ncactions; // needs to be accessible from handle_event()

bool EXPANSION_ATTACHED = false;
bool KEEP_RUNNING = 1;
bool WIIMOTE_ACC_MOUSE = false;
bool WIIMOTE_IR_MOUSE = false;
/* USED FOR CRAPPY CODE
bool CC_LJS_MOUSE = false; */

int mouse_x = 0;
int mouse_y = 0;
int STICK_SENSITIVITY = 10;
int ACCELEROMETER_SENSITIVITY = 10; // Lower is more sensitive

wiimote** wiimotes;

int main(int argc, char *argv[])
{
// Check usage
if (argc != 2)
	{
	cout << "Correct usage:\n\txwii profile_filename" << endl;
	return 1;
	}

// Initialise X Low-level variables
display = XOpenDisplay(0);
if(display == NULL)
	return 1;
winRoot = XDefaultRootWindow(display);

// Initialise wiiuse library stuff
const char* version = wiiuse_startup("./wiiuse.so");
if (!version) 
    { 
    fprintf(stderr, "Failed to load wiiuse library (wiiuse.so).\n");
    return 0;
    }

enum { WIIMOTE_ID_1 = 1};
int ids[] = { WIIMOTE_ID_1 };

wiimotes = wiiuse_init(1, ids, handle_event, handle_ctrl_status, handle_disconnect);
cout << "Press 1 + 2 to put the Wii Remote in discoverable mode." << endl;
int found = wiiuse_find(wiimotes, 1, 5);

int connected = wiiuse_connect(wiimotes, 1);
if (connected)
    {
    printf("Connected to %i wiimotes (of %i found).\n", connected, found);
    }
else
    { 
    printf("Failed to connect to any wiimote.\n");
    return 0;
    }

// Initialize ActionSet
ifstream profile;
profile.open(argv[1], ios::in);
if (!profile.is_open())
    {
    cout << "Could not open specified config file for reading.  Exiting..." << endl;
    exit(1);
    }
actions = ActionSet(profile);
profile.close();
profile.open(argv[1], ios::in);
if (!profile.is_open())
    {
    cout << "Could not open specified config file for reading.  Exiting..." << endl;
    exit(1);
    }
ccactions = CC_ActionSet(profile);
profile.close();
profile.open(argv[1], ios::in);
if (!profile.is_open())
    {
    cout << "Could not open specified config file for reading.  Exiting..." << endl;
    exit(1);
    }
ncactions = NC_ActionSet(profile);
profile.close();

wiiuse_motion_sensing(wiimotes[0], 1);

// Begin Main Input Loop
long int new_time, old_time = get_time();
while (KEEP_RUNNING)
    {
    wiiuse_poll(wiimotes, 1);
    new_time = get_time();
    update_led_timer(wiimotes[0], new_time );
    if ((new_time - old_time) > FRAME_DURATION)
        {
        process_mouse_instructions();
        old_time = new_time;
        }
    }

// End program
wiiuse_shutdown();
XCloseDisplay(display);
return 0;
}

long int get_time()
{
static struct timeval mytime;
gettimeofday(&mytime, NULL);
long int returnable = (mytime.tv_sec * 1000000) + mytime.tv_usec;
return returnable;
}

void handle_event(struct wiimote_t* wm)
{
if (WIIMOTE_ACC_MOUSE)
    {
    if (wm->orient.roll > ROLL_THRESHOLD || wm->orient.roll < (-1 * ROLL_THRESHOLD))
        mouse_x = int(wm->orient.roll) / ACCELEROMETER_SENSITIVITY;
    else
        mouse_x = 0;
    if (wm->orient.pitch > PITCH_THRESHOLD || wm->orient.pitch < (-1 * PITCH_THRESHOLD))
        mouse_y = -1 * (int(wm->orient.pitch) / ACCELEROMETER_SENSITIVITY);
    else
        mouse_y = 0;
    }

if (WIIMOTE_IR_MOUSE)
    {
    mouse_x = wm->ir.x;
    mouse_y = wm->ir.y;
    }

/* Variables used to track state of non-button inputs using button-like interface (eg. shoulder buttons) */
static bool L_PRESSED = false;
static bool R_PRESSED = false;
static bool LS_UP_PRESSED = false;
static bool LS_DOWN_PRESSED = false;
static bool LS_LEFT_PRESSED = false;
static bool LS_RIGHT_PRESSED = false;
static bool RS_UP_PRESSED = false;
static bool RS_DOWN_PRESSED = false;
static bool RS_LEFT_PRESSED = false;
static bool RS_RIGHT_PRESSED = false;
static bool MIDFLICK = false;
static bool NC_UP_PRESSED = false;
static bool NC_DOWN_PRESSED = false;
static bool NC_LEFT_PRESSED = false;
static bool NC_RIGHT_PRESSED = false;
static bool NC_MIDFLICK = false;
float floor = 0.0, ceiling = 0.0;
int theta, quadrant;

InputState istate = InputState();

if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_A))		{ istate.a = true; istate.a_type = PRESSED;}
if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_B))		{ istate.b = true; istate.b_type = PRESSED; }
if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_UP))		{ istate.up = true; istate.up_type = PRESSED; }
if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_DOWN))	{ istate.down = true; istate.down_type = PRESSED; }
if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_LEFT))	{ istate.left = true; istate.left_type = PRESSED; }
if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_RIGHT))	{ istate.right = true; istate.right_type = PRESSED; }
if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_MINUS))	{ istate.minus = true; istate.minus_type = PRESSED; }
if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_PLUS))	{ istate.plus = true; istate.plus_type = PRESSED; }
if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_ONE))	{ istate.one = true; istate.one_type = PRESSED; }
if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_TWO))	{ istate.two = true; istate.two_type = PRESSED; }
if (IS_JUST_PRESSED(wm, WIIMOTE_BUTTON_HOME))	{ istate.home = true; istate.home_type = PRESSED; }

if (IS_RELEASED(wm, WIIMOTE_BUTTON_A))		{ istate.a = true; istate.a_type = RELEASED;}
if (IS_RELEASED(wm, WIIMOTE_BUTTON_B))		{ istate.b = true; istate.b_type = RELEASED; };
if (IS_RELEASED(wm, WIIMOTE_BUTTON_UP))		{ istate.up = true; istate.up_type = RELEASED; }
if (IS_RELEASED(wm, WIIMOTE_BUTTON_DOWN))	{ istate.down = true; istate.down_type = RELEASED; }
if (IS_RELEASED(wm, WIIMOTE_BUTTON_LEFT))	{ istate.left = true; istate.left_type = RELEASED; }
if (IS_RELEASED(wm, WIIMOTE_BUTTON_RIGHT))	{ istate.right = true; istate.right_type = RELEASED; }
if (IS_RELEASED(wm, WIIMOTE_BUTTON_MINUS))	{ istate.minus = true; istate.minus_type = RELEASED; }
if (IS_RELEASED(wm, WIIMOTE_BUTTON_PLUS))	{ istate.plus = true; istate.plus_type = RELEASED; }
if (IS_RELEASED(wm, WIIMOTE_BUTTON_ONE))	{ istate.one = true; istate.one_type = RELEASED; }
if (IS_RELEASED(wm, WIIMOTE_BUTTON_TWO))	{ istate.two = true; istate.two_type = RELEASED; }
if (IS_RELEASED(wm, WIIMOTE_BUTTON_HOME))	{ istate.home = true; istate.home_type = RELEASED; }

if (
    (wm->gforce.x > FLICK_THRESHOLD ||
    wm->gforce.y > FLICK_THRESHOLD ||   // if a flick has occurred
    wm->gforce.z > FLICK_THRESHOLD ||
    (-1 * wm->gforce.x) < (-1 * FLICK_THRESHOLD) ||
    (-1 * wm->gforce.y) < (-1 * FLICK_THRESHOLD) ||
    (-1 * wm->gforce.z) < (-1 * FLICK_THRESHOLD))
    &&
    !MIDFLICK)
    {
    istate.flick = true; // so that flick event is acted upon
    istate.flick_type = PRESSED;
    MIDFLICK = true; // so that another flick event isn't acted upon until another flick happens
    }
else if (
    (wm->gforce.x <= FLICK_THRESHOLD &&
    wm->gforce.y <= FLICK_THRESHOLD &&   // if all gforces are below threshold
    wm->gforce.z <= FLICK_THRESHOLD &&
    (-1 * wm->gforce.x) >= (-1 * FLICK_THRESHOLD) &&
    (-1 * wm->gforce.y) >= (-1 * FLICK_THRESHOLD) &&
    (-1 * wm->gforce.z) >= (-1 * FLICK_THRESHOLD))
    &&
    MIDFLICK)
    {
    istate.flick = true; // so that flick event is acted upon
    istate.flick_type = RELEASED;
    MIDFLICK = false;
    }

xwii_process_input(istate, actions);

// Classic Controller and Nunchuk Events
if (EXPANSION_ATTACHED)
    {
    /* Classic Controller or Wii Guitar */
    CC_InputState ccistate = CC_InputState(); 

    struct classic_ctrl_t* cc = (classic_ctrl_t*)&wm->exp.classic;
    if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_A))		{ ccistate.a = true; ccistate.a_type = PRESSED; }
    if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_B))		{ ccistate.b = true; ccistate.b_type = PRESSED; }
    if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_X))		{ ccistate.x = true; ccistate.x_type = PRESSED; }
    if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_Y))		{ ccistate.y = true; ccistate.y_type = PRESSED; }
    if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_HOME))	{ ccistate.home = true; ccistate.home_type = PRESSED; }
	if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_ZL))	{ ccistate.zl = true; ccistate.zl_type = PRESSED; }
	if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_ZR))	{ ccistate.zr = true; ccistate.zr_type = PRESSED; }
	if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_LEFT))	{ ccistate.d_left = true; ccistate.d_left_type = PRESSED; }
	if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_UP))	{ ccistate.d_up = true; ccistate.d_up_type = PRESSED; }
	if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_RIGHT))	{ ccistate.d_right = true; ccistate.d_right_type = PRESSED; }
	if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_DOWN))	{ ccistate.d_down = true; ccistate.d_down_type = PRESSED; }
	if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_MINUS))	{ ccistate.minus = true; ccistate.minus_type = PRESSED; }
	if (IS_JUST_PRESSED(cc, CLASSIC_CTRL_BUTTON_PLUS))	{ ccistate.plus = true; ccistate.plus_type = PRESSED; }

    if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_A))		    { ccistate.a = true; ccistate.a_type = RELEASED; }
    if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_B))	    	{ ccistate.b = true; ccistate.b_type = RELEASED; }
    if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_X))		    { ccistate.x = true; ccistate.x_type = RELEASED; }
    if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_Y))		    { ccistate.y = true; ccistate.y_type = RELEASED; }
    if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_HOME))	    { ccistate.home = true; ccistate.home_type = RELEASED; }
	if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_ZL))	    { ccistate.zl = true; ccistate.zl_type = RELEASED; }
	if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_ZR))	    { ccistate.zr = true; ccistate.zr_type = RELEASED; }
	if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_LEFT))	    { ccistate.d_left = true; ccistate.d_left_type = RELEASED; }
	if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_UP))	    { ccistate.d_up = true; ccistate.d_up_type = RELEASED; }
	if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_RIGHT))	    { ccistate.d_right = true; ccistate.d_right_type = RELEASED; }
	if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_DOWN))	    { ccistate.d_down = true; ccistate.d_down_type = RELEASED; }
	if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_MINUS))	    { ccistate.minus = true; ccistate.minus_type = RELEASED; }
	if (IS_RELEASED(cc, CLASSIC_CTRL_BUTTON_PLUS))	    { ccistate.plus = true; ccistate.plus_type = RELEASED; }
	
	/* Classic Controller Sticks and Shoulder Buttons */
	if (cc->l_shoulder >= (double(SHOULDER_MAGNITUDE_THRESHOLD) / DENOMINATOR) && L_PRESSED == false) 
	    { 
	    ccistate.l = true;
	    ccistate.l_type = PRESSED;
	    L_PRESSED = true;
	    }
	else if (cc->l_shoulder < (double(SHOULDER_MAGNITUDE_THRESHOLD) / DENOMINATOR) && L_PRESSED == true)
	    {
	    ccistate.l = true;
	    ccistate.l_type = RELEASED;
	    L_PRESSED = false;
	    }
	if (cc->r_shoulder >= (double(SHOULDER_MAGNITUDE_THRESHOLD) / DENOMINATOR) && R_PRESSED == false) 
	    { 
	    ccistate.r = true;
	    ccistate.r_type = PRESSED;
	    R_PRESSED = true;
	    }
	else if (cc->r_shoulder < (double(SHOULDER_MAGNITUDE_THRESHOLD) / DENOMINATOR) && R_PRESSED == true)
	    {
	    ccistate.r = true;
	    ccistate.r_type = RELEASED;
	    R_PRESSED = false;
	    }

    
    if (cc->ljs.mag >= (double(STICK_MAGNITUDE_THRESHOLD) / DENOMINATOR))
        {
        floor = cc->ljs.ang - STICK_ANGLE_THRESHOLD;
        ceiling = cc->ljs.ang + STICK_ANGLE_THRESHOLD;
        if (floor <= 90 && 90 <= ceiling && LS_RIGHT_PRESSED == false)
            {
            ccistate.ls_right = true;
            ccistate.ls_right_type = PRESSED;
            LS_RIGHT_PRESSED = true;
            }
        else if (!(floor <= 90 && 90 <= ceiling) && LS_RIGHT_PRESSED == true)
            {
            ccistate.ls_right = true;
            ccistate.ls_right_type = RELEASED;
            LS_RIGHT_PRESSED = false;
            }
        if (floor <= 180 && 180 <= ceiling && LS_DOWN_PRESSED == false)
            {
            ccistate.ls_down = true;
            ccistate.ls_down_type = PRESSED;
            LS_DOWN_PRESSED = true;
            }
        else if (!(floor <= 180 && 180 <= ceiling) && LS_DOWN_PRESSED == true)
            {
            ccistate.ls_down = true;
            ccistate.ls_down_type = RELEASED;
            LS_DOWN_PRESSED = false;
            }
        if (floor <= 270 && 270 <= ceiling && LS_LEFT_PRESSED == false)
            {
            ccistate.ls_left = true;
            ccistate.ls_left_type = PRESSED;
            LS_LEFT_PRESSED = true;
            }
        else if (!(floor <= 270 && 270 <= ceiling) && LS_LEFT_PRESSED == true)
            {
            ccistate.ls_left = true;
            ccistate.ls_left_type = RELEASED;
            LS_LEFT_PRESSED = false;
            }
        if ((floor <= 0 || ceiling >= 360) && LS_UP_PRESSED == false)
            {
            ccistate.ls_up = true;
            ccistate.ls_up_type = PRESSED;
            LS_UP_PRESSED = true;
            }
        else if (!(floor <= 0 || ceiling >= 360) && LS_UP_PRESSED == true)
            {
            ccistate.ls_up = true;
            ccistate.ls_up_type = RELEASED;
            LS_UP_PRESSED = false;
            }
        }
    else
        {
        if (LS_RIGHT_PRESSED)
            { 
            LS_RIGHT_PRESSED = false; 
            ccistate.ls_right = true;
            ccistate.ls_right_type = RELEASED;
            }
        if (LS_DOWN_PRESSED)
            { 
            LS_DOWN_PRESSED = false; 
            ccistate.ls_down = true;
            ccistate.ls_down_type = RELEASED;
            }
        if (LS_LEFT_PRESSED)
            { 
            LS_LEFT_PRESSED = false; 
            ccistate.ls_left = true;
            ccistate.ls_left_type = RELEASED;
            }
        if (LS_UP_PRESSED)
            { 
            LS_UP_PRESSED = false; 
            ccistate.ls_up = true;
            ccistate.ls_up_type = RELEASED;
            }
        }
    
    if (cc->rjs.mag >= (double(STICK_MAGNITUDE_THRESHOLD) / DENOMINATOR))
        {
        floor = cc->rjs.ang - STICK_ANGLE_THRESHOLD;
        ceiling = cc->rjs.ang + STICK_ANGLE_THRESHOLD;
        if (floor <= 90 && 90 <= ceiling && RS_RIGHT_PRESSED == false)
            {
            ccistate.rs_right = true;
            ccistate.rs_right_type = PRESSED;
            RS_RIGHT_PRESSED = true;
            }
        else if (!(floor <= 90 && 90 <= ceiling) && RS_RIGHT_PRESSED == true)
            {
            ccistate.rs_right = true;
            ccistate.rs_right_type = RELEASED;
            RS_RIGHT_PRESSED = false;
            }
        if (floor <= 180 && 180 <= ceiling && RS_DOWN_PRESSED == false)
            {
            ccistate.rs_down = true;
            ccistate.rs_down_type = PRESSED;
            RS_DOWN_PRESSED = true;
            }
        else if (!(floor <= 180 && 180 <= ceiling) && RS_DOWN_PRESSED == true)
            {
            ccistate.rs_down = true;
            ccistate.rs_down_type = RELEASED;
            RS_DOWN_PRESSED = false;
            }
        if (floor <= 270 && 270 <= ceiling && RS_LEFT_PRESSED == false)
            {
            ccistate.rs_left = true;
            ccistate.rs_left_type = PRESSED;
            RS_LEFT_PRESSED = true;
            }
        else if (!(floor <= 270 && 270 <= ceiling) && RS_LEFT_PRESSED == true)
            {
            ccistate.rs_left = true;
            ccistate.rs_left_type = RELEASED;
            RS_LEFT_PRESSED = false;
            }
        if ((floor <= 0 || ceiling >= 360) && RS_UP_PRESSED == false)
            {
            ccistate.rs_up = true;
            ccistate.rs_up_type = PRESSED;
            RS_UP_PRESSED = true;
            }
        else if (!(floor <= 0 || ceiling >= 360) && RS_UP_PRESSED == true)
            {
            ccistate.rs_up = true;
            ccistate.rs_up_type = RELEASED;
            RS_UP_PRESSED = false;
            }
        }
    else
        {
        if (RS_RIGHT_PRESSED)
            { 
            RS_RIGHT_PRESSED = false; 
            ccistate.rs_right = true;
            ccistate.rs_right_type = RELEASED;
            }
        if (RS_DOWN_PRESSED)
            { 
            RS_DOWN_PRESSED = false; 
            ccistate.rs_down = true;
            ccistate.rs_down_type = RELEASED;
            }
        if (RS_LEFT_PRESSED)
            { 
            RS_LEFT_PRESSED = false; 
            ccistate.rs_left = true;
            ccistate.rs_left_type = RELEASED;
            }
        if (RS_UP_PRESSED)
            { 
            RS_UP_PRESSED = false; 
            ccistate.rs_up = true;
            ccistate.rs_up_type = RELEASED;
            }
        }
		
    
    xwii_process_CC_input(ccistate, ccactions);
    
    /* I would make the classic and nunchuk mutually exclusive but I can't reliably determine which is plugged in yet */ 
    /* Nunchuk */
    struct nunchuk_t* nc = (nunchuk_t*)&wm->exp.nunchuk;
    NC_InputState ncistate = NC_InputState();
    
    /* Fill out the NC_InputState object */
    if (IS_JUST_PRESSED(nc, NUNCHUK_BUTTON_C))		{ ncistate.c = true; ncistate.c_type = PRESSED; }
    if (IS_JUST_PRESSED(nc, NUNCHUK_BUTTON_Z))		{ ncistate.z = true; ncistate.z_type = PRESSED; }
    
    if (IS_RELEASED(nc, NUNCHUK_BUTTON_C))  		{ ncistate.c = true; ncistate.c_type = RELEASED; }
    if (IS_RELEASED(nc, NUNCHUK_BUTTON_Z))  		{ ncistate.z = true; ncistate.z_type = RELEASED; }

    if (nc->js.mag >= (double(STICK_MAGNITUDE_THRESHOLD) / DENOMINATOR))
        {
        floor = nc->js.ang - STICK_ANGLE_THRESHOLD;
        ceiling = nc->js.ang + STICK_ANGLE_THRESHOLD;
        if (floor <= 90 && 90 <= ceiling && NC_RIGHT_PRESSED == false)
            {
            ncistate.stick_right = true;
            ncistate.stick_right_type = PRESSED;
            NC_RIGHT_PRESSED = true;
            }
        else if (!(floor <= 90 && 90 <= ceiling) && NC_RIGHT_PRESSED == true)
            {
            ncistate.stick_right = true;
            ncistate.stick_right_type = RELEASED;
            NC_RIGHT_PRESSED = false;
            }
        if (floor <= 180 && 180 <= ceiling && NC_DOWN_PRESSED == false)
            {
            ncistate.stick_down = true;
            ncistate.stick_down_type = PRESSED;
            NC_DOWN_PRESSED = true;
            }
        else if (!(floor <= 180 && 180 <= ceiling) && NC_DOWN_PRESSED == true)
            {
            ncistate.stick_down = true;
            ncistate.stick_down_type = RELEASED;
            NC_DOWN_PRESSED = false;
            }
        if (floor <= 270 && 270 <= ceiling && NC_LEFT_PRESSED == false)
            {
            ncistate.stick_left = true;
            ncistate.stick_left_type = PRESSED;
            NC_LEFT_PRESSED = true;
            }
        else if (!(floor <= 270 && 270 <= ceiling) && NC_LEFT_PRESSED == true)
            {
            ncistate.stick_left = true;
            ncistate.stick_left_type = RELEASED;
            NC_LEFT_PRESSED = false;
            }
        if ((floor <= 0 || ceiling >= 360) && NC_UP_PRESSED == false)
            {
            ncistate.stick_up = true;
            ncistate.stick_up_type = PRESSED;
            NC_UP_PRESSED = true;
            }
        else if (!(floor <= 0 || ceiling >= 360) && NC_UP_PRESSED == true)
            {
            ncistate.stick_up = true;
            ncistate.stick_up_type = RELEASED;
            NC_UP_PRESSED = false;
            }
        }
    else
        {
        if (NC_RIGHT_PRESSED)
            { 
            NC_RIGHT_PRESSED = false; 
            ncistate.stick_right = true;
            ncistate.stick_right_type = RELEASED;
            }
        if (NC_DOWN_PRESSED)
            { 
            NC_DOWN_PRESSED = false; 
            ncistate.stick_down = true;
            ncistate.stick_down_type = RELEASED;
            }
        if (NC_LEFT_PRESSED)
            { 
            NC_LEFT_PRESSED = false; 
            ncistate.stick_left = true;
            ncistate.stick_left_type = RELEASED;
            }
        if (NC_UP_PRESSED)
            { 
            NC_UP_PRESSED = false; 
            ncistate.stick_up = true;
            ncistate.stick_up_type = RELEASED;
            }
        }    

    if (
        (nc->gforce.x > NC_FLICK_THRESHOLD ||
        nc->gforce.y > NC_FLICK_THRESHOLD ||   // if a flick has occurred
        nc->gforce.z > NC_FLICK_THRESHOLD ||
        (-1 * nc->gforce.x) < (-1 * NC_FLICK_THRESHOLD) ||
        (-1 * nc->gforce.y) < (-1 * NC_FLICK_THRESHOLD) ||
        (-1 * nc->gforce.z) < (-1 * NC_FLICK_THRESHOLD))
        &&
        !NC_MIDFLICK)
        {
        ncistate.flick = true; // so that flick event is acted upon
        ncistate.flick_type = PRESSED;
        NC_MIDFLICK = true; // so that another flick event isn't acted upon until another flick happens
        }
    else if (
        (nc->gforce.x <= NC_FLICK_THRESHOLD &&
        nc->gforce.y <= NC_FLICK_THRESHOLD &&   // if all gforces are below threshold
        nc->gforce.z <= NC_FLICK_THRESHOLD &&
        (-1 * nc->gforce.x) >= (-1 * NC_FLICK_THRESHOLD) &&
        (-1 * nc->gforce.y) >= (-1 * NC_FLICK_THRESHOLD) &&
        (-1 * nc->gforce.z) >= (-1 * NC_FLICK_THRESHOLD))
        &&
        NC_MIDFLICK)
        {
        ncistate.flick = true; // so that flick event is acted upon
        ncistate.flick_type = RELEASED;
        NC_MIDFLICK = false;
        }

    xwii_process_NC_input(ncistate, ncactions);
    }
return;
}

void handle_ctrl_status(struct wiimote_t* wm, int attachment, int speaker, int ir, int led[4], float battery_level)
{
if (attachment)
    {
    EXPANSION_ATTACHED = true;
    }
else
    EXPANSION_ATTACHED = false;
cout << "Wiimote status changed." << endl;
return;
}

void process_mouse_instructions()
{
if (WIIMOTE_ACC_MOUSE)
XIMovePointer(mouse_x, mouse_y, true); // relative movement
else if (WIIMOTE_IR_MOUSE)
XIMovePointer(mouse_x, mouse_y, false); // absolute movement
return;
}

// start timer
long int liLedTimerStart = 0;
bool bLedTimerStarted = false;

// short cut for each led
short sLeds[] = {WIIMOTE_LED_1, WIIMOTE_LED_2, WIIMOTE_LED_3, WIIMOTE_LED_4};

// timer delay
long int liBaseDelay  = 30;
long int liTimerDelay = 1000000 * liBaseDelay;
long int liLastTimer = 0;

/**
 * Updates the time of the wiimote, by setting a binary clock
 */
void update_led_timer(wiimote_t* wm, long int time )
{
  // check, if the timer was started
  if( bLedTimerStarted == false )
  {
    liLedTimerStart = time;
    bLedTimerStarted = true;
  }

  // update the clock?
  if( abs(time - liLastTimer) <= liTimerDelay )
  {
    // no!
    return;
  }

  // update the timer
  liLastTimer = time;

  // calculate the time passed ( in seconds )
  int iTimePassed = abs((liLedTimerStart - time) / (1000000 * liBaseDelay));
  
  // update the leds to reflect the time passed (look at the bits of the time)
  short sBinaryTime[] = {0,0,0,0};
  int iLeds = 0;
  for( int i = 0; i < 4; i++)
  {
    sBinaryTime[i] = abs((iTimePassed >> i) % 2 != 0);
    iLeds |= sBinaryTime[i] * sLeds[3-i];
  }

cout << "timepassed: "<< iTimePassed << " x " << liBaseDelay << " seconds" << endl; 

  // update the leds
  wiiuse_set_leds(wm, iLeds);

  // make a realy short rumble
  wiiuse_rumble(wm,true);
  usleep(61250);
  wiiuse_rumble(wm,false);
}
